@@ -20,5 +20,5 @@ int main(int argc, char **argv)
 void bb_show_usage(void)
 {
 	fputs_stdout(APPLET_full_usage "\n");
-	exit(EXIT_FAILURE);
+	exit_FAILURE();
 }
