@@ -16,7 +16,7 @@
  * a diagnostic message and an error return.
  */
 //config:config LOGNAME
-//config:	bool "logname (1.1 kb)"
+//config:	bool "logname (1.4 kb)"
 //config:	default y
 //config:	help
 //config:	logname is used to print the current user's login name.
@@ -56,5 +56,5 @@ int logname_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 		return fflush_all();
 	}
 
-	bb_perror_msg_and_die("getlogin");
+	bb_simple_perror_msg_and_die("getlogin");
 }
