@@ -27,6 +27,8 @@
 //usage:#if ENABLE_DESKTOP
 //usage:#define od_trivial_usage
 //usage:       "[-abcdfhilovxs] [-t TYPE] [-A RADIX] [-N SIZE] [-j SKIP] [-S MINSTR] [-w WIDTH] [FILE]..."
+// We also support -BDOHXIL, but they are not documented in coreutils 9.1
+// manpage/help, so don't show them either.
 // We don't support:
 // ... [FILE] [[+]OFFSET[.][b]]
 // Support is buggy for:
@@ -43,26 +45,33 @@ enum {
 	OPT_b = 1 << 3,
 	OPT_c = 1 << 4,
 	OPT_d = 1 << 5,
-	OPT_f = 1 << 6,
-	OPT_h = 1 << 7,
-	OPT_i = 1 << 8,
-	OPT_j = 1 << 9,
-	OPT_l = 1 << 10,
-	OPT_o = 1 << 11,
-	OPT_t = 1 << 12,
+	OPT_D = 1 << 6, /* undocumented in coreutils 9.1 */
+	OPT_f = 1 << 7,
+	OPT_h = 1 << 8,
+	OPT_H = 1 << 9, /* undocumented in coreutils 9.1 */
+	OPT_i = 1 << 10,
+	OPT_I = 1 << 11, /* undocumented in coreutils 9.1 */
+	OPT_j = 1 << 12,
+	OPT_l = 1 << 13,
+	OPT_L = 1 << 14, /* undocumented in coreutils 9.1 */
+	OPT_o = 1 << 15,
+	OPT_O = 1 << 16, /* undocumented in coreutils 9.1 */
+	OPT_B = 1 << 17, /* undocumented synonym to -o */
+	OPT_t = 1 << 18,
 	/* When zero and two or more consecutive blocks are equal, format
 	   only the first block and output an asterisk alone on the following
 	   line to indicate that identical blocks have been elided: */
-	OPT_v = 1 << 13,
-	OPT_x = 1 << 14,
-	OPT_s = 1 << 15,
-	OPT_S = 1 << 16,
-	OPT_w = 1 << 17,
-	OPT_traditional = (1 << 18) * ENABLE_LONG_OPTS,
+	OPT_v = 1 << 19,
+	OPT_x = 1 << 20,
+	OPT_X = 1 << 21, /* undocumented in coreutils 9.1 */
+	OPT_s = 1 << 22,
+	OPT_S = 1 << 23,
+	OPT_w = 1 << 24,
+	OPT_traditional = (1 << 25) * ENABLE_LONG_OPTS,
 };
 
 #define OD_GETOPT32() getopt32long(argv, \
-	"A:N:abcdfhij:lot:*vxsS:w:+:", od_longopts, \
+	"A:N:abcdDfhHiIj:lLoOBt:*vxXsS:w:+:", od_longopts, \
 	/* -w with optional param */ \
 	/* -S was -s and also had optional parameter */ \
 	/* but in coreutils 6.3 it was renamed and now has */ \
@@ -1239,20 +1248,29 @@ int od_main(int argc UNUSED_PARAM, char **argv)
 	if (opt & OPT_N) {
 		max_bytes_to_format = xstrtooff_sfx(str_N, 0, bkm_suffixes);
 	}
+
 	if (opt & OPT_a) decode_format_string("a");
 	if (opt & OPT_b) decode_format_string("oC");
 	if (opt & OPT_c) decode_format_string("c");
 	if (opt & OPT_d) decode_format_string("u2");
+	if (opt & OPT_D) decode_format_string("uI");
 	if (opt & OPT_f) decode_format_string("fF");
-	if (opt & OPT_h) decode_format_string("x2");
-	if (opt & OPT_i) decode_format_string("d2");
+	if (opt & (OPT_h|OPT_x)) decode_format_string("x2");
+	if (opt & (OPT_H|OPT_X)) decode_format_string("xI");
+	/* -I,L,l: depend on word width of the arch (what is "long"?) */
+#if ULONG_MAX > 0xffffffff
+	if (opt & OPT_i) decode_format_string("dI");
+	if (opt & (OPT_I|OPT_l|OPT_L)) decode_format_string("dL");
+#else
+	/* 32-bit arch: -I,L,l are the same as -i */
+	if (opt & (OPT_i|OPT_I|OPT_l|OPT_L)) decode_format_string("dI");
+#endif
 	if (opt & OPT_j) n_bytes_to_skip = xstrtooff_sfx(str_j, 0, bkm_suffixes);
-	if (opt & OPT_l) decode_format_string("d4");
-	if (opt & OPT_o) decode_format_string("o2");
+	if (opt & (OPT_o|OPT_B)) decode_format_string("o2");
+	if (opt & OPT_O) decode_format_string("oI");
 	while (lst_t) {
 		decode_format_string(llist_pop(&lst_t));
 	}
-	if (opt & OPT_x) decode_format_string("x2");
 	if (opt & OPT_s) decode_format_string("d2");
 	if (opt & OPT_S) {
 		G.string_min = xstrtou_sfx(str_S, 0, bkm_suffixes);
