@@ -7,7 +7,7 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config READLINK
-//config:	bool "readlink (4 kb)"
+//config:	bool "readlink (4.8 kb)"
 //config:	default y
 //config:	help
 //config:	This program reads a symbolic link and returns the name
@@ -25,12 +25,14 @@
 //kbuild:lib-$(CONFIG_READLINK) += readlink.o
 
 //usage:#define readlink_trivial_usage
-//usage:	IF_FEATURE_READLINK_FOLLOW("[-fnv] ") "FILE"
+//usage:	IF_FEATURE_READLINK_FOLLOW("[-fnv] ")
+//usage:	IF_NOT_FEATURE_READLINK_FOLLOW("[-n] ")
+//usage:	"FILE"
 //usage:#define readlink_full_usage "\n\n"
-//usage:       "Display the value of a symlink"
-//usage:	IF_FEATURE_READLINK_FOLLOW( "\n"
-//usage:     "\n	-f	Canonicalize by following all symlinks"
+//usage:       "Display the value of a symlink" "\n"
 //usage:     "\n	-n	Don't add newline"
+//usage:	IF_FEATURE_READLINK_FOLLOW(
+//usage:     "\n	-f	Canonicalize by following all symlinks"
 //usage:     "\n	-v	Verbose"
 //usage:	)
 
@@ -66,35 +68,27 @@ int readlink_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int readlink_main(int argc UNUSED_PARAM, char **argv)
 {
 	char *buf;
-	char *fname;
+	unsigned opt;
 
-	IF_FEATURE_READLINK_FOLLOW(
-		unsigned opt;
-		/* We need exactly one non-option argument.  */
-		opt = getopt32(argv, "^" "fnvsq" "\0" "=1");
-		fname = argv[optind];
-	)
-	IF_NOT_FEATURE_READLINK_FOLLOW(
-		const unsigned opt = 0;
-		if (argc != 2) bb_show_usage();
-		fname = argv[1];
-	)
+	/* -n must use bit 0 (see printf below) */
+	opt = getopt32(argv, "^" "n" IF_FEATURE_READLINK_FOLLOW("fvsq")
+                       "\0" "=1");
 
 	/* compat: coreutils readlink reports errors silently via exit code */
 	if (!(opt & 4)) /* not -v */
 		logmode = LOGMODE_NONE;
 
 	/* NOFORK: only one alloc is allowed; must free */
-	if (opt & 1) { /* -f */
-		buf = xmalloc_realpath_coreutils(fname);
+	if (opt & 2) { /* -f */
+		buf = xmalloc_realpath_coreutils(argv[optind]);
 	} else {
-		buf = xmalloc_readlink_or_warn(fname);
+		buf = xmalloc_readlink_or_warn(argv[optind]);
 	}
 
 	if (!buf)
 		return EXIT_FAILURE;
-	printf((opt & 2) ? "%s" : "%s\n", buf);
+	printf("%s%s", buf, &"\n"[opt & 1]);
 	free(buf);
 
-	fflush_stdout_and_exit(EXIT_SUCCESS);
+	fflush_stdout_and_exit_SUCCESS();
 }
