@@ -11,7 +11,7 @@
  * Size reduction.
  */
 //config:config RM
-//config:	bool "rm (5.4 kb)"
+//config:	bool "rm (5.5 kb)"
 //config:	default y
 //config:	help
 //config:	rm is used to remove files or directories.
@@ -62,7 +62,7 @@ int rm_main(int argc UNUSED_PARAM, char **argv)
 			const char *base = bb_get_last_path_component_strip(*argv);
 
 			if (DOT_OR_DOTDOT(base)) {
-				bb_error_msg("can't remove '.' or '..'");
+				bb_simple_error_msg("can't remove '.' or '..'");
 			} else if (remove_file(*argv, flags) >= 0) {
 				continue;
 			}
