@@ -10,7 +10,7 @@
  * Special for busybox ported by Vladimir Oleynik <dzo@simtreas.ru> 2001
  */
 //config:config STTY
-//config:	bool "stty (8.9 kb)"
+//config:	bool "stty (9.2 kb)"
 //config:	default y
 //config:	help
 //config:	stty is used to change and print terminal line settings.
@@ -487,7 +487,7 @@ static const char mode_name[] ALIGN1 =
 #undef MI_ENTRY
 #define MI_ENTRY(N,T,F,B,M) { T, F, M, B },
 
-static const struct mode_info mode_info[] = {
+static const struct mode_info mode_info[] ALIGN4 = {
 	/* This should be verbatim cut-n-paste copy of the above MI_ENTRYs */
 	MI_ENTRY("evenp",    combination, REV        | OMIT, 0,          0 )
 	MI_ENTRY("parity",   combination, REV        | OMIT, 0,          0 )
@@ -855,7 +855,7 @@ static void wrapf(const char *message, ...)
 			}
 		}
 	}
-	fputs(buf, stdout);
+	fputs_stdout(buf);
 	G.current_col += buflen;
 	if (buf[buflen-1] == '\n')
 		G.current_col = 0;
@@ -905,7 +905,7 @@ static void display_window_size(int fancy)
 	}
 }
 
-static const struct suffix_mult stty_suffixes[] = {
+static const struct suffix_mult stty_suffixes[] ALIGN_SUFFIX = {
 	{ "b",  512 },
 	{ "k", 1024 },
 	{ "B", 1024 },
