@@ -51,7 +51,7 @@
  * diff -u /tmp/std_find /tmp/bb_find && echo Identical
  */
 //config:config FIND
-//config:	bool "find (14 kb)"
+//config:	bool "find (16 kb)"
 //config:	default y
 //config:	help
 //config:	find is used to search your system to find specified files.
@@ -67,21 +67,53 @@
 //config:	interpreted by other programs.
 //config:
 //config:config FEATURE_FIND_MTIME
-//config:	bool "Enable -mtime: modified time matching"
+//config:	bool "Enable -mtime: modification time matching"
 //config:	default y
 //config:	depends on FIND
 //config:	help
 //config:	Allow searching based on the modification time of
 //config:	files, in days.
 //config:
+//config:config FEATURE_FIND_ATIME
+//config:	bool "Enable -atime: access time matching"
+//config:	default y
+//config:	depends on FEATURE_FIND_MTIME
+//config:	help
+//config:	Allow searching based on the access time of
+//config:	files, in days.
+//config:
+//config:config FEATURE_FIND_CTIME
+//config:	bool "Enable -ctime: status change timestamp matching"
+//config:	default y
+//config:	depends on FEATURE_FIND_MTIME
+//config:	help
+//config:	Allow searching based on the status change timestamp of
+//config:	files, in days.
+//config:
 //config:config FEATURE_FIND_MMIN
-//config:	bool "Enable -mmin: modified time matching by minutes"
+//config:	bool "Enable -mmin: modification time matching by minutes"
 //config:	default y
 //config:	depends on FIND
 //config:	help
 //config:	Allow searching based on the modification time of
 //config:	files, in minutes.
 //config:
+//config:config FEATURE_FIND_AMIN
+//config:	bool "Enable -amin: access time matching by minutes"
+//config:	default y
+//config:	depends on FEATURE_FIND_MMIN
+//config:	help
+//config:	Allow searching based on the access time of
+//config:	files, in minutes.
+//config:
+//config:config FEATURE_FIND_CMIN
+//config:	bool "Enable -cmin: status change timestamp matching by minutes"
+//config:	default y
+//config:	depends on FEATURE_FIND_MMIN
+//config:	help
+//config:	Allow searching based on the status change timestamp of
+//config:	files, in minutes.
+//config:
 //config:config FEATURE_FIND_PERM
 //config:	bool "Enable -perm: permissions matching"
 //config:	default y
@@ -123,6 +155,13 @@
 //config:	default y
 //config:	depends on FIND
 //config:
+//config:config FEATURE_FIND_SAMEFILE
+//config:	bool "Enable -samefile: reference file matching"
+//config:	default y
+//config:	depends on FIND
+//config:	help
+//config:	Support the 'find -samefile' option for searching by a reference file.
+//config:
 //config:config FEATURE_FIND_EXEC
 //config:	bool "Enable -exec: execute commands"
 //config:	default y
@@ -141,6 +180,13 @@
 //config:	Without this option, -exec + is a synonym for -exec ;
 //config:	(IOW: it works correctly, but without expected speedup)
 //config:
+//config:config FEATURE_FIND_EXEC_OK
+//config:	bool "Enable -ok: execute confirmed commands"
+//config:	default y
+//config:	depends on FEATURE_FIND_EXEC
+//config:	help
+//config:	Support the 'find -ok' option which prompts before executing.
+//config:
 //config:config FEATURE_FIND_USER
 //config:	bool "Enable -user: username/uid matching"
 //config:	default y
@@ -296,16 +342,31 @@
 //usage:     "\n	-mtime DAYS	mtime is greater than (+N), less than (-N),"
 //usage:     "\n			or exactly N days in the past"
 //usage:	)
+//usage:	IF_FEATURE_FIND_ATIME(
+//usage:     "\n	-atime DAYS	atime +N/-N/N days in the past"
+//usage:	)
+//usage:	IF_FEATURE_FIND_CTIME(
+//usage:     "\n	-ctime DAYS	ctime +N/-N/N days in the past"
+//usage:	)
 //usage:	IF_FEATURE_FIND_MMIN(
 //usage:     "\n	-mmin MINS	mtime is greater than (+N), less than (-N),"
 //usage:     "\n			or exactly N minutes in the past"
 //usage:	)
+//usage:	IF_FEATURE_FIND_AMIN(
+//usage:     "\n	-amin MINS	atime +N/-N/N minutes in the past"
+//usage:	)
+//usage:	IF_FEATURE_FIND_CMIN(
+//usage:     "\n	-cmin MINS	ctime +N/-N/N minutes in the past"
+//usage:	)
 //usage:	IF_FEATURE_FIND_NEWER(
 //usage:     "\n	-newer FILE	mtime is more recent than FILE's"
 //usage:	)
 //usage:	IF_FEATURE_FIND_INUM(
 //usage:     "\n	-inum N		File has inode number N"
 //usage:	)
+//usage:	IF_FEATURE_FIND_SAMEFILE(
+//usage:     "\n	-samefile FILE	File is same as FILE"
+//usage:	)
 //usage:	IF_FEATURE_FIND_USER(
 //usage:     "\n	-user NAME/ID	File is owned by given user"
 //usage:	)
@@ -341,6 +402,9 @@
 //usage:	IF_FEATURE_FIND_EXEC_PLUS(
 //usage:     "\n	-exec CMD ARG + Run CMD with {} replaced by list of file names"
 //usage:	)
+//usage:	IF_FEATURE_FIND_EXEC_OK(
+//usage:     "\n	-ok CMD ARG ;   Prompt and run CMD with {} replaced"
+//usage:	)
 //usage:	IF_FEATURE_FIND_DELETE(
 //usage:     "\n	-delete		Delete current file/directory. Turns on -depth option"
 //usage:	)
@@ -396,10 +460,11 @@ IF_FEATURE_FIND_PRINT0( ACTS(print0))
 IF_FEATURE_FIND_TYPE(   ACTS(type,  int type_mask;))
 IF_FEATURE_FIND_EXECUTABLE(ACTS(executable))
 IF_FEATURE_FIND_PERM(   ACTS(perm,  char perm_char; mode_t perm_mask;))
-IF_FEATURE_FIND_MTIME(  ACTS(mtime, char mtime_char; unsigned mtime_days;))
-IF_FEATURE_FIND_MMIN(   ACTS(mmin,  char mmin_char; unsigned mmin_mins;))
+IF_FEATURE_FIND_MTIME(  ACTS(mtime, unsigned char time_type; unsigned char mtime_char; unsigned mtime_days;))
+IF_FEATURE_FIND_MMIN(   ACTS(mmin,  unsigned char time_type; unsigned char mmin_char; unsigned mmin_mins;))
 IF_FEATURE_FIND_NEWER(  ACTS(newer, time_t newer_mtime;))
 IF_FEATURE_FIND_INUM(   ACTS(inum,  ino_t inode_num;))
+IF_FEATURE_FIND_SAMEFILE(ACTS(samefile, ino_t inode_num; dev_t device;))
 IF_FEATURE_FIND_USER(   ACTS(user,  uid_t uid;))
 IF_FEATURE_FIND_SIZE(   ACTS(size,  char size_char; off_t size;))
 IF_FEATURE_FIND_CONTEXT(ACTS(context, security_context_t context;))
@@ -412,6 +477,9 @@ IF_FEATURE_FIND_EXEC(   ACTS(exec,
 				char **exec_argv; /* -exec ARGS */
 				unsigned *subst_count;
 				int exec_argc; /* count of ARGS */
+				IF_FEATURE_FIND_EXEC_OK(
+				int ok;		/* -ok */
+				)
 				IF_FEATURE_FIND_EXEC_PLUS(
 					/*
 					 * filelist is NULL if "exec ;"
@@ -618,30 +686,61 @@ ACTF(perm)
 	return (statbuf->st_mode & 07777) == ap->perm_mask;
 }
 #endif
+
+#if 0 \
+ || ENABLE_FEATURE_FIND_AMIN  \
+ || ENABLE_FEATURE_FIND_ATIME \
+ || ENABLE_FEATURE_FIND_CMIN  \
+ || ENABLE_FEATURE_FIND_CTIME \
+ || ENABLE_FEATURE_FIND_MMIN  \
+ || ENABLE_FEATURE_FIND_MTIME
+static int time_cmp(const struct stat *statbuf, unsigned type_and_char, time_t N_from_user, unsigned unit)
+{
+	time_t ftime, file_age;
+
+	ftime = statbuf->st_mtime;
+# if ENABLE_FEATURE_FIND_ATIME || ENABLE_FEATURE_FIND_CTIME
+#  if ENABLE_FEATURE_FIND_ATIME
+	if ((type_and_char >> 8) == 'a')
+		ftime = statbuf->st_atime;
+#  endif
+#  if ENABLE_FEATURE_FIND_CTIME
+	if ((type_and_char >> 8) == 'c')
+		ftime = statbuf->st_ctime;
+#  endif
+	type_and_char &= 0xff;
+# endif
+	file_age = time(NULL) - ftime;
+	N_from_user *= unit;
+	switch (type_and_char) {
+	case '+': return file_age >= N_from_user + unit;
+	case '-': return file_age < N_from_user;
+	/* just numeric time */
+	default:  return file_age >= N_from_user && file_age < N_from_user + unit;
+	}
+}
+#endif
+
 #if ENABLE_FEATURE_FIND_MTIME
 ACTF(mtime)
 {
-	time_t file_age = time(NULL) - statbuf->st_mtime;
-	time_t mtime_secs = ap->mtime_days * 24*60*60;
-	if (ap->mtime_char == '+')
-		return file_age >= mtime_secs + 24*60*60;
-	if (ap->mtime_char == '-')
-		return file_age < mtime_secs;
-	/* just numeric mtime */
-	return file_age >= mtime_secs && file_age < (mtime_secs + 24*60*60);
+	return time_cmp(statbuf,
+# if ENABLE_FEATURE_FIND_ATIME || ENABLE_FEATURE_FIND_CTIME
+			(ap->time_type << 8) |
+# endif
+				ap->mtime_char,
+			ap->mtime_days, 24*60*60);
 }
 #endif
 #if ENABLE_FEATURE_FIND_MMIN
 ACTF(mmin)
 {
-	time_t file_age = time(NULL) - statbuf->st_mtime;
-	time_t mmin_secs = ap->mmin_mins * 60;
-	if (ap->mmin_char == '+')
-		return file_age >= mmin_secs + 60;
-	if (ap->mmin_char == '-')
-		return file_age < mmin_secs;
-	/* just numeric mmin */
-	return file_age >= mmin_secs && file_age < (mmin_secs + 60);
+	return time_cmp(statbuf,
+# if ENABLE_FEATURE_FIND_ATIME || ENABLE_FEATURE_FIND_CTIME
+			(ap->time_type << 8) |
+# endif
+				ap->mmin_char,
+			ap->mmin_mins, 60);
 }
 #endif
 #if ENABLE_FEATURE_FIND_NEWER
@@ -656,6 +755,13 @@ ACTF(inum)
 	return (statbuf->st_ino == ap->inode_num);
 }
 #endif
+#if ENABLE_FEATURE_FIND_SAMEFILE
+ACTF(samefile)
+{
+	return statbuf->st_ino == ap->inode_num &&
+	       statbuf->st_dev == ap->device;
+}
+#endif
 #if ENABLE_FEATURE_FIND_EXEC
 static int do_exec(action_exec *ap, const char *fileName)
 {
@@ -709,10 +815,24 @@ static int do_exec(action_exec *ap, const char *fileName)
 	}
 # endif
 
+# if ENABLE_FEATURE_FIND_EXEC_OK
+	if (ap->ok) {
+		for (i = 0; argv[i]; i++)
+			fprintf(stderr, "%s ", argv[i]);
+		fprintf(stderr, "?");
+		if (!bb_ask_y_confirmation()) {
+			rc = 1; /* "false" */
+			goto not_ok;
+		}
+	}
+# endif
 	rc = spawn_and_wait(argv);
 	if (rc < 0)
 		bb_simple_perror_msg(argv[0]);
 
+# if ENABLE_FEATURE_FIND_EXEC_OK
+    not_ok:
+# endif
 	i = 0;
 	while (argv[i])
 		free(argv[i++]);
@@ -1027,6 +1147,7 @@ static action*** parse_params(char **argv)
 	IF_FEATURE_FIND_DELETE( PARM_delete    ,)
 	IF_FEATURE_FIND_EMPTY(	PARM_empty     ,)
 	IF_FEATURE_FIND_EXEC(   PARM_exec      ,)
+	IF_FEATURE_FIND_EXEC_OK(PARM_ok        ,)
 	IF_FEATURE_FIND_EXECUTABLE(PARM_executable,)
 	IF_FEATURE_FIND_PAREN(  PARM_char_brace,)
 	/* All options/actions starting from here require argument */
@@ -1043,9 +1164,14 @@ static action*** parse_params(char **argv)
 	IF_FEATURE_FIND_TYPE(   PARM_type      ,)
 	IF_FEATURE_FIND_PERM(   PARM_perm      ,)
 	IF_FEATURE_FIND_MTIME(  PARM_mtime     ,)
+	IF_FEATURE_FIND_ATIME(  PARM_atime     ,)
+	IF_FEATURE_FIND_CTIME(  PARM_ctime     ,)
 	IF_FEATURE_FIND_MMIN(   PARM_mmin      ,)
+	IF_FEATURE_FIND_AMIN(   PARM_amin      ,)
+	IF_FEATURE_FIND_CMIN(   PARM_cmin      ,)
 	IF_FEATURE_FIND_NEWER(  PARM_newer     ,)
 	IF_FEATURE_FIND_INUM(   PARM_inum      ,)
+	IF_FEATURE_FIND_SAMEFILE(PARM_samefile ,)
 	IF_FEATURE_FIND_USER(   PARM_user      ,)
 	IF_FEATURE_FIND_GROUP(  PARM_group     ,)
 	IF_FEATURE_FIND_SIZE(   PARM_size      ,)
@@ -1073,6 +1199,7 @@ static action*** parse_params(char **argv)
 	IF_FEATURE_FIND_DELETE( "-delete\0" )
 	IF_FEATURE_FIND_EMPTY(	"-empty\0"  )
 	IF_FEATURE_FIND_EXEC(   "-exec\0"   )
+	IF_FEATURE_FIND_EXEC_OK("-ok\0"     )
 	IF_FEATURE_FIND_EXECUTABLE("-executable\0")
 	IF_FEATURE_FIND_PAREN(  "(\0"       )
 	/* All options/actions starting from here require argument */
@@ -1087,9 +1214,14 @@ static action*** parse_params(char **argv)
 	IF_FEATURE_FIND_TYPE(   "-type\0"   )
 	IF_FEATURE_FIND_PERM(   "-perm\0"   )
 	IF_FEATURE_FIND_MTIME(  "-mtime\0"  )
+	IF_FEATURE_FIND_ATIME(  "-atime\0"  )
+	IF_FEATURE_FIND_CTIME(  "-ctime\0"  )
 	IF_FEATURE_FIND_MMIN(   "-mmin\0"   )
+	IF_FEATURE_FIND_AMIN(   "-amin\0"   )
+	IF_FEATURE_FIND_CMIN(   "-cmin\0"   )
 	IF_FEATURE_FIND_NEWER(  "-newer\0"  )
 	IF_FEATURE_FIND_INUM(   "-inum\0"   )
+	IF_FEATURE_FIND_SAMEFILE("-samefile\0")
 	IF_FEATURE_FIND_USER(   "-user\0"   )
 	IF_FEATURE_FIND_GROUP(  "-group\0"  )
 	IF_FEATURE_FIND_SIZE(   "-size\0"   )
@@ -1248,23 +1380,27 @@ static action*** parse_params(char **argv)
 		}
 #endif
 #if ENABLE_FEATURE_FIND_EXEC
-		else if (parm == PARM_exec) {
+		else if (parm == PARM_exec IF_FEATURE_FIND_EXEC_OK(|| parm == PARM_ok)) {
 			int i;
 			action_exec *ap;
 			IF_FEATURE_FIND_EXEC_PLUS(int all_subst = 0;)
 			dbg("%d", __LINE__);
 			G.need_print = 0;
 			ap = ALLOC_ACTION(exec);
+			IF_FEATURE_FIND_EXEC_OK(ap->ok = (parm == PARM_ok);)
 			ap->exec_argv = ++argv; /* first arg after -exec */
 			/*ap->exec_argc = 0; - ALLOC_ACTION did it */
 			while (1) {
 				if (!*argv) /* did not see ';' or '+' until end */
-					bb_error_msg_and_die(bb_msg_requires_arg, "-exec");
+					bb_error_msg_and_die(bb_msg_requires_arg, arg);
 				// find -exec echo Foo ">{}<" ";"
 				// executes "echo Foo >FILENAME<",
 				// find -exec echo Foo ">{}<" "+"
 				// executes "echo Foo FILENAME1 FILENAME2 FILENAME3...".
-				if ((argv[0][0] == ';' || argv[0][0] == '+')
+				if ((argv[0][0] == ';'
+				     || (argv[0][0] == '+' IF_FEATURE_FIND_EXEC_OK(&& parm != PARM_ok))
+						/* -ok CMD + syntax is not accepted, only with ';' */
+				    )
 				 && argv[0][1] == '\0'
 				) {
 # if ENABLE_FEATURE_FIND_EXEC_PLUS
@@ -1373,19 +1509,39 @@ static action*** parse_params(char **argv)
 		}
 #endif
 #if ENABLE_FEATURE_FIND_MTIME
-		else if (parm == PARM_mtime) {
+		else if (parm == PARM_mtime
+# if ENABLE_FEATURE_FIND_ATIME
+		 || parm == PARM_atime
+# endif
+# if ENABLE_FEATURE_FIND_CTIME
+		 || parm == PARM_ctime
+# endif
+		) {
 			action_mtime *ap;
 			dbg("%d", __LINE__);
 			ap = ALLOC_ACTION(mtime);
+# if ENABLE_FEATURE_FIND_AMIN || ENABLE_FEATURE_FIND_CMIN
+			ap->time_type = arg[1];
+# endif
 			ap->mtime_char = arg1[0];
 			ap->mtime_days = xatoul(plus_minus_num(arg1));
 		}
 #endif
 #if ENABLE_FEATURE_FIND_MMIN
-		else if (parm == PARM_mmin) {
+		else if (parm == PARM_mmin
+# if ENABLE_FEATURE_FIND_AMIN
+		 || parm == PARM_amin
+# endif
+# if ENABLE_FEATURE_FIND_CMIN
+		 || parm == PARM_cmin
+# endif
+		) {
 			action_mmin *ap;
 			dbg("%d", __LINE__);
 			ap = ALLOC_ACTION(mmin);
+# if ENABLE_FEATURE_FIND_AMIN || ENABLE_FEATURE_FIND_CMIN
+			ap->time_type = arg[1];
+# endif
 			ap->mmin_char = arg1[0];
 			ap->mmin_mins = xatoul(plus_minus_num(arg1));
 		}
@@ -1408,6 +1564,21 @@ static action*** parse_params(char **argv)
 			ap->inode_num = xatoul(arg1);
 		}
 #endif
+#if ENABLE_FEATURE_FIND_SAMEFILE
+		else if (parm == PARM_samefile) {
+			action_samefile *ap;
+			struct stat stbuf;
+			dbg("%d", __LINE__);
+			if (G.recurse_flags & (ACTION_FOLLOWLINKS |
+					       ACTION_FOLLOWLINKS_L0))
+				xstat(arg1, &stbuf);
+			else if (lstat(arg1, &stbuf))
+				bb_perror_msg_and_die("can't stat '%s'", arg1);
+			ap = ALLOC_ACTION(samefile);
+			ap->inode_num = stbuf.st_ino;
+			ap->device = stbuf.st_dev;
+		}
+#endif
 #if ENABLE_FEATURE_FIND_USER
 		else if (parm == PARM_user) {
 			action_user *ap;
