@@ -31,6 +31,9 @@
 # define ADJTIME_PATH "/etc/adjtime"
 #endif
 
+#define STR1(s) #s
+#define STR(s)  STR1(s)
+
 INSERT
 
 #define busybox_notes_usage \
