@@ -296,7 +296,7 @@ Special characters:
 
 /* Code here assumes that 'unsigned' is at least 32 bits wide */
 
-const char *const bb_argv_dash[] = { "-", NULL };
+const char *const bb_argv_dash[] ALIGN_PTR = { "-", NULL };
 
 enum {
 	PARAM_STRING,
