@@ -268,10 +268,12 @@ pid_t FAST_FUNC fork_or_rexec(char **argv)
 	/* fflush_all(); ? - so far all callers had no buffered output to flush */
 
 	pid = xvfork();
-	if (pid) /* parent */
-		return pid;
-	/* child - re-exec ourself */
-	re_exec(argv);
+	if (pid == 0) /* child - re-exec ourself */
+		re_exec(argv); /* NORETURN */
+
+	/* parent */
+	argv[0][0] &= 0x7f; /* undo re_rexec() damage */
+	return pid;
 }
 #endif
 
@@ -294,9 +296,12 @@ void FAST_FUNC bb_daemonize_or_rexec(int flags, char **argv)
 	}
 
 	if (flags & DAEMON_DEVNULL_STDIO) {
-		xdup2(fd, 0);
-		xdup2(fd, 1);
-		xdup2(fd, 2);
+		if (flags & DAEMON_DEVNULL_STDIN)
+			xdup2(fd, 0);
+		if (flags & DAEMON_DEVNULL_OUTERR) {
+			xdup2(fd, 1);
+			xdup2(fd, 2);
+		}
 	} else {
 		/* have 0,1,2 open at least to /dev/null */
 		while ((unsigned)fd < 2)
@@ -308,7 +313,7 @@ void FAST_FUNC bb_daemonize_or_rexec(int flags, char **argv)
 		/* fflush_all(); - add it in fork_or_rexec() if necessary */
 
 		if (fork_or_rexec(argv))
-			_exit(EXIT_SUCCESS); /* parent */
+			_exit_SUCCESS(); /* parent */
 		/* if daemonizing, detach from stdio & ctty */
 		setsid();
 		dup2(fd, 0);
@@ -320,7 +325,7 @@ void FAST_FUNC bb_daemonize_or_rexec(int flags, char **argv)
 //			 * Prevent this: stop being a session leader.
 //			 */
 //			if (fork_or_rexec(argv))
-//				_exit(EXIT_SUCCESS); /* parent */
+//				_exit_SUCCESS(); /* parent */
 //		}
 	}
 	while (fd > 2) {
