@@ -9,7 +9,7 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config ADDGROUP
-//config:	bool "addgroup (8.6 kb)"
+//config:	bool "addgroup (8.8 kb)"
 //config:	default y
 //config:	select LONG_OPTS
 //config:	help
@@ -102,7 +102,7 @@ static void new_group(char *group, gid_t gid)
 	/* add entry to group */
 	p = xasprintf("x:%u:", (unsigned) gr.gr_gid);
 	if (update_passwd(bb_path_group_file, group, p, NULL) < 0)
-		exit(EXIT_FAILURE);
+		exit_FAILURE();
 	if (ENABLE_FEATURE_CLEAN_UP)
 		free(p);
 #if ENABLE_FEATURE_SHADOWPASSWDS
