@@ -9,27 +9,28 @@
  * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 //config:config DELUSER
-//config:	bool "deluser"
+//config:	bool "deluser (9.3 kb)"
 //config:	default y
 //config:	help
-//config:	  Utility for deleting a user account.
+//config:	Utility for deleting a user account.
 //config:
 //config:config DELGROUP
-//config:	bool "delgroup"
+//config:	bool "delgroup (6.6 kb)"
 //config:	default y
 //config:	help
-//config:	  Utility for deleting a group account.
+//config:	Utility for deleting a group account.
 //config:
 //config:config FEATURE_DEL_USER_FROM_GROUP
 //config:	bool "Support removing users from groups"
 //config:	default y
 //config:	depends on DELGROUP
 //config:	help
-//config:	  If called with two non-option arguments, deluser
-//config:	  or delgroup will remove an user from a specified group.
+//config:	If called with two non-option arguments, deluser
+//config:	or delgroup will remove an user from a specified group.
 
-//applet:IF_DELUSER(APPLET(deluser, BB_DIR_USR_SBIN, BB_SUID_DROP))
-//applet:IF_DELGROUP(APPLET_ODDNAME(delgroup, deluser, BB_DIR_USR_SBIN, BB_SUID_DROP, delgroup))
+//                   APPLET_NOEXEC:name      main     location         suid_type     help
+//applet:IF_DELUSER( APPLET_NOEXEC(deluser,  deluser, BB_DIR_USR_SBIN, BB_SUID_DROP, deluser))
+//applet:IF_DELGROUP(APPLET_NOEXEC(delgroup, deluser, BB_DIR_USR_SBIN, BB_SUID_DROP, delgroup))
 
 //kbuild:lib-$(CONFIG_DELUSER) += deluser.o
 //kbuild:lib-$(CONFIG_DELGROUP) += deluser.o
@@ -67,16 +68,15 @@ int deluser_main(int argc, char **argv)
 #else
 	int opt_delhome = 0;
 	if (do_deluser) {
-		applet_long_options =
-			"remove-home\0" No_argument "\xff";
-		opt_delhome = getopt32(argv, "");
+		opt_delhome = getopt32long(argv, "",
+				"remove-home\0" No_argument "\xff");
 		argv += opt_delhome;
 		argc -= opt_delhome;
 	}
 #endif
 
 	if (geteuid() != 0)
-		bb_error_msg_and_die(bb_msg_perm_denied_are_you_root);
+		bb_simple_error_msg_and_die(bb_msg_perm_denied_are_you_root);
 
 	name = argv[1];
 	member = NULL;
@@ -99,8 +99,14 @@ int deluser_main(int argc, char **argv)
 			pfile = bb_path_passwd_file;
 			if (ENABLE_FEATURE_SHADOWPASSWDS)
 				sfile = bb_path_shadow_file;
-			if (opt_delhome)
-				remove_file(pw->pw_dir, FILEUTILS_RECUR);
+			if (opt_delhome) {
+				struct stat st;
+
+				/* Make sure home is an actual directory before
+				 * removing it (e.g. users with /dev/null as home) */
+				if (stat(pw->pw_dir, &st) == 0 && S_ISDIR(st.st_mode))
+					remove_file(pw->pw_dir, FILEUTILS_RECUR);
+			}
 		} else {
 			struct group *gr;
  do_delgroup:
