@@ -9,31 +9,31 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config I2CGET
-//config:	bool "i2cget (5.5 kb)"
+//config:	bool "i2cget (5.7 kb)"
 //config:	default y
 //config:	help
 //config:	Read from I2C/SMBus chip registers.
 //config:
 //config:config I2CSET
-//config:	bool "i2cset (6.7 kb)"
+//config:	bool "i2cset (6.9 kb)"
 //config:	default y
 //config:	help
 //config:	Set I2C registers.
 //config:
 //config:config I2CDUMP
-//config:	bool "i2cdump (7.1 kb)"
+//config:	bool "i2cdump (7.2 kb)"
 //config:	default y
 //config:	help
 //config:	Examine I2C registers.
 //config:
 //config:config I2CDETECT
-//config:	bool "i2cdetect (7.1 kb)"
+//config:	bool "i2cdetect (7.3 kb)"
 //config:	default y
 //config:	help
 //config:	Detect I2C chips.
 //config:
 //config:config I2CTRANSFER
-//config:	bool "i2ctransfer (4.0 kb)"
+//config:	bool "i2ctransfer (5.5 kb)"
 //config:	default y
 //config:	help
 //config:	Send user-defined I2C messages in one transfer.
@@ -107,6 +107,7 @@ static ALWAYS_INLINE void *itoptr(int i)
 	return (void*)(intptr_t)i;
 }
 
+#if ENABLE_I2CGET || ENABLE_I2CSET || ENABLE_I2CDUMP || ENABLE_I2CDETECT
 static int32_t i2c_smbus_access(int fd, char read_write, uint8_t cmd,
 				int size, union i2c_smbus_data *data)
 {
@@ -131,6 +132,7 @@ static int32_t i2c_smbus_read_byte(int fd)
 
 	return data.byte;
 }
+#endif
 
 #if ENABLE_I2CGET || ENABLE_I2CSET || ENABLE_I2CDUMP
 static int32_t i2c_smbus_write_byte(int fd, uint8_t val)
@@ -1149,12 +1151,12 @@ static void NORETURN list_i2c_busses_and_exit(void)
 		/* Simple version for ISA chips. */
 		snprintf(path, NAME_MAX, "%s/%s/name",
 			 i2cdev_path, de->d_name);
-		fp = fopen(path, "r");
+		fp = fopen_for_read(path);
 		if (fp == NULL) {
 			snprintf(path, NAME_MAX,
 				 "%s/%s/device/name",
 				 i2cdev_path, de->d_name);
-			fp = fopen(path, "r");
+			fp = fopen_for_read(path);
 		}
 
 		/* Non-ISA chips require the hard-way. */
@@ -1175,7 +1177,7 @@ static void NORETURN list_i2c_busses_and_exit(void)
 						 "%s/%s/device/%s/name",
 						 i2cdev_path, de->d_name,
 						 subde->d_name);
-					fp = fopen(path, "r");
+					fp = fopen_for_read(path);
 					break;
 				}
 			}
@@ -1212,7 +1214,7 @@ static void NORETURN list_i2c_busses_and_exit(void)
 		}
 	}
 
-	exit(EXIT_SUCCESS);
+	exit_SUCCESS();
 }
 
 static void NORETURN no_support(const char *cmd)
@@ -1394,7 +1396,7 @@ static void check_i2c_func(int fd)
 }
 
 //usage:#define i2ctransfer_trivial_usage
-//usage:       "[-fay] I2CBUS {rLENGTH[@ADDR] | wLENGTH[@ADDR] DATA...}..."
+//usage:       "[-fay] I2CBUS { rLENGTH[@ADDR] | wLENGTH[@ADDR] DATA...}..."
 //usage:#define i2ctransfer_full_usage "\n\n"
 //usage:       "Read/write I2C data in one transfer"
 //usage:     "\n"
