@@ -7,7 +7,7 @@
  * known bugs: can't deal with alpha ranges
  */
 //config:config MAKEDEVS
-//config:	bool "makedevs (9.2 kb)"
+//config:	bool "makedevs (9.4 kb)"
 //config:	default y
 //config:	help
 //config:	'makedevs' is a utility used to create a batch of devices with
@@ -181,7 +181,7 @@ int makedevs_main(int argc UNUSED_PARAM, char **argv)
 {
 	parser_t *parser;
 	char *line = (char *)"-";
-	int ret = EXIT_SUCCESS;
+	exitcode_t ret = EXIT_SUCCESS;
 
 	getopt32(argv, "^" "d:" "\0" "=1", &line);
 	argv += optind;
