@@ -7,7 +7,7 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config STRINGS
-//config:	bool "strings (4.6 kb)"
+//config:	bool "strings (4.8 kb)"
 //config:	default y
 //config:	help
 //config:	strings prints the printable character sequences for each file
@@ -18,7 +18,7 @@
 //kbuild:lib-$(CONFIG_STRINGS) += strings.o
 
 //usage:#define strings_trivial_usage
-//usage:       "[-fo] [-t o/d/x] [-n LEN] [FILE]..."
+//usage:       "[-fo] [-t o|d|x] [-n LEN] [FILE]..."
 //usage:#define strings_full_usage "\n\n"
 //usage:       "Display printable strings in a binary file\n"
 //We usually don't bother user with "nop" options. They work, but are not shown:
@@ -26,7 +26,7 @@
 //unimplemented alternative is -d: Only strings from initialized, loaded data sections
 //usage:     "\n	-f		Precede strings with filenames"
 //usage:     "\n	-o		Precede strings with octal offsets"
-//usage:     "\n	-t o/d/x	Precede strings with offsets in base 8/10/16"
+//usage:     "\n	-t o|d|x	Precede strings with offsets in base 8/10/16"
 //usage:     "\n	-n LEN		At least LEN characters form a string (default 4)"
 
 #include "libbb.h"
@@ -40,7 +40,8 @@
 int strings_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int strings_main(int argc UNUSED_PARAM, char **argv)
 {
-	int n, c, status = EXIT_SUCCESS;
+	int n, c;
+	exitcode_t status = EXIT_SUCCESS;
 	unsigned count;
 	off_t offset;
 	FILE *file;
@@ -92,7 +93,7 @@ int strings_main(int argc UNUSED_PARAM, char **argv)
 						if (option_mask32 & (PRINT_OFFSET | PRINT_RADIX)) {
 							printf(radix_fmt, offset - n);
 						}
-						fputs(string, stdout);
+						fputs_stdout(string);
 					}
 					count++;
 				}
