@@ -10,7 +10,7 @@
  * Heavily modified for busybox by Erik Andersen <andersen@codepoet.org>
  */
 //config:config TIME
-//config:	bool "time (6.8 kb)"
+//config:	bool "time (8.1 kb)"
 //config:	default y
 //config:	help
 //config:	The time command runs the specified program with the given arguments.
@@ -33,6 +33,13 @@
 
 #include "libbb.h"
 
+#ifndef HAVE_WAIT3
+static pid_t wait3(int *status, int options, struct rusage *rusage)
+{
+	return wait4(-1, status, options, rusage);
+}
+#endif
+
 /* Information on the resources used by a child process.  */
 typedef struct {
 	int waitstatus;
@@ -104,6 +111,7 @@ static void printargv(char *const *argv)
 	} while (*++argv);
 }
 
+#ifdef UNUSED
 /* Return the number of kilobytes corresponding to a number of pages PAGES.
    (Actually, we use it to convert pages*ticks into kilobytes*ticks.)
 
@@ -129,6 +137,7 @@ static unsigned long ptok(const unsigned pagesize, const unsigned long pages)
 	return tmp / 1024;      /* then smaller.  */
 }
 #undef pagesize
+#endif /* UNUSED */
 
 /* summarize: Report on the system use of a command.
 
@@ -217,35 +226,30 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 		}
 
 		switch (*fmt) {
-#ifdef NOT_NEEDED
-		/* Handle literal char */
-		/* Usually we optimize for size, but there is a limit
-		 * for everything. With this we do a lot of 1-byte writes */
-		default:
-			bb_putchar(*fmt);
-			break;
-#endif
-
 		case '%':
 			switch (*++fmt) {
-#ifdef NOT_NEEDED_YET
-		/* Our format strings do not have these */
-		/* and we do not take format str from user */
 			default:
-				bb_putchar('%');
+				/* Unknown %<char> is printed as "?<char>" */
+				bb_putchar('?');
+				if (!*fmt) {
+					/* Trailing -f '...%' prints "...?" but NOT newline */
+					goto ret;
+				}
 				/*FALLTHROUGH*/
 			case '%':
-				if (!*fmt) goto ret;
 				bb_putchar(*fmt);
 				break;
-#endif
 			case 'C':	/* The command that got timed.  */
 				printargv(command);
 				break;
 			case 'D':	/* Average unshared data size.  */
+				/* (linux kernel sets ru_idrss/isrss/ixrss to 0,
+				 * docs say the value is in kbytes, so ptok() is wrong) */
 				printf("%lu",
-					(ptok(pagesize, (UL) resp->ru.ru_idrss) +
-					 ptok(pagesize, (UL) resp->ru.ru_isrss)) / cpu_ticks);
+					(/*ptok(pagesize,*/ (UL) resp->ru.ru_idrss +
+						(UL) resp->ru.ru_isrss
+					) / cpu_ticks
+				);
 				break;
 			case 'E': {	/* Elapsed real (wall clock) time.  */
 				unsigned seconds = resp->elapsed_ms / 1000;
@@ -268,13 +272,17 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 				printf("%lu", resp->ru.ru_inblock);
 				break;
 			case 'K':	/* Average mem usage == data+stack+text.  */
+				/* (linux kernel sets ru_idrss/isrss/ixrss to 0,
+				 * docs say the value is in kbytes, so ptok() is wrong) */
 				printf("%lu",
-					(ptok(pagesize, (UL) resp->ru.ru_idrss) +
-					 ptok(pagesize, (UL) resp->ru.ru_isrss) +
-					 ptok(pagesize, (UL) resp->ru.ru_ixrss)) / cpu_ticks);
+					(/*ptok(pagesize,*/ (UL) resp->ru.ru_idrss +
+						(UL) resp->ru.ru_isrss +
+						(UL) resp->ru.ru_ixrss
+					) / cpu_ticks
+				);
 				break;
 			case 'M':	/* Maximum resident set size.  */
-				printf("%lu", ptok(pagesize, (UL) resp->ru.ru_maxrss));
+				printf("%lu", (UL) resp->ru.ru_maxrss);
 				break;
 			case 'O':	/* Outputs.  */
 				printf("%lu", resp->ru.ru_oublock);
@@ -327,7 +335,7 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 				printf("%lu", resp->ru.ru_nswap);
 				break;
 			case 'X':	/* Average shared text size.  */
-				printf("%lu", ptok(pagesize, (UL) resp->ru.ru_ixrss) / cpu_ticks);
+				printf("%lu", /*ptok(pagesize,*/ (UL) resp->ru.ru_ixrss / cpu_ticks);
 				break;
 			case 'Z':	/* Page size.  */
 				printf("%u", pagesize);
@@ -344,7 +352,7 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 				printf("%lu", resp->ru.ru_nsignals);
 				break;
 			case 'p':	/* Average stack segment.  */
-				printf("%lu", ptok(pagesize, (UL) resp->ru.ru_isrss) / cpu_ticks);
+				printf("%lu", /*ptok(pagesize,*/ (UL) resp->ru.ru_isrss / cpu_ticks);
 				break;
 			case 'r':	/* Incoming socket messages received.  */
 				printf("%lu", resp->ru.ru_msgrcv);
@@ -353,7 +361,7 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 				printf("%lu", resp->ru.ru_msgsnd);
 				break;
 			case 't':	/* Average resident set size.  */
-				printf("%lu", ptok(pagesize, (UL) resp->ru.ru_idrss) / cpu_ticks);
+				printf("%lu", /*ptok(pagesize,*/ (UL) resp->ru.ru_idrss / cpu_ticks);
 				break;
 			case 'w':	/* Voluntary context switches.  */
 				printf("%lu", resp->ru.ru_nvcsw);
@@ -364,14 +372,21 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 			}
 			break;
 
-#ifdef NOT_NEEDED_YET
-		case '\\':		/* Format escape.  */
+		default: /* *fmt is '\': format escape */
 			switch (*++fmt) {
 			default:
+				/* Unknown \<char> is printed as "?\<char>" */
+				bb_putchar('?');
 				bb_putchar('\\');
+				if (!*fmt) {
+					/* Trailing -f '...\': GNU time 1.9 prints
+					 * "...?\COMMAND" (it's probably a bug).
+					 */
+					puts(command[0]);
+					goto ret;
+				}
 				/*FALLTHROUGH*/
 			case '\\':
-				if (!*fmt) goto ret;
 				bb_putchar(*fmt);
 				break;
 			case 't':
@@ -382,12 +397,11 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
 				break;
 			}
 			break;
-#endif
 		}
 		++fmt;
 	}
- /* ret: */
 	bb_putchar('\n');
+ ret: ;
 }
 
 /* Run command CMD and return statistics on it.
@@ -421,7 +435,7 @@ int time_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int time_main(int argc UNUSED_PARAM, char **argv)
 {
 	resource_t res;
-	/* $TIME has lowest prio (-v,-p,-f FMT overrride it) */
+	/* $TIME has lowest prio (-v,-p,-f FMT override it) */
 	const char *output_format = getenv("TIME") ? : default_format;
 	char *output_filename;
 	int output_fd;
