@@ -102,6 +102,12 @@
 //config:	help
 //config:	HTTP server.
 //config:
+//config:config FEATURE_HTTPD_PORT_DEFAULT
+//config:	int "Default port"
+//config:	default 80
+//config:	range 1 65535
+//config:	depends on HTTPD
+//config:
 //config:config FEATURE_HTTPD_RANGES
 //config:	bool "Support 'Ranges:' header"
 //config:	default y
@@ -268,9 +274,9 @@
 //usage:#define httpd_full_usage "\n\n"
 //usage:       "Listen for incoming HTTP requests\n"
 //usage:     "\n	-i		Inetd mode"
-//usage:     "\n	-f		Don't daemonize"
+//usage:     "\n	-f		Run in foreground"
 //usage:     "\n	-v[v]		Verbose"
-//usage:     "\n	-p [IP:]PORT	Bind to IP:PORT (default *:80)"
+//usage:     "\n	-p [IP:]PORT	Bind to IP:PORT (default *:"STR(CONFIG_FEATURE_HTTPD_PORT_DEFAULT)")"
 //usage:	IF_FEATURE_HTTPD_SETUID(
 //usage:     "\n	-u USER[:GRP]	Set uid/gid after binding to port")
 //usage:	IF_FEATURE_HTTPD_BASIC_AUTH(
@@ -316,6 +322,9 @@
 
 #define HEADER_READ_TIMEOUT 60
 
+#define STR1(s) #s
+#define STR(s) STR1(s)
+
 static const char DEFAULT_PATH_HTTPD_CONF[] ALIGN1 = "/etc";
 static const char HTTPD_CONF[] ALIGN1 = "httpd.conf";
 static const char HTTP_200[] ALIGN1 = "HTTP/1.1 200 OK\r\n";
@@ -542,7 +551,7 @@ enum {
 	SET_PTR_TO_GLOBALS(xzalloc(sizeof(G))); \
 	IF_FEATURE_HTTPD_BASIC_AUTH(g_realm = "Web Server Authentication";) \
 	IF_FEATURE_HTTPD_RANGES(range_start = -1;) \
-	bind_addr_or_port = "80"; \
+	bind_addr_or_port = STR(CONFIG_FEATURE_HTTPD_PORT_DEFAULT); \
 	index_page = index_html; \
 	file_size = -1; \
 } while (0)
@@ -1116,7 +1125,7 @@ static void send_headers(unsigned responseNum)
 			"Connection: close\r\n",
 			responseNum, responseString
 #if ENABLE_FEATURE_HTTPD_DATE
-			,date_str
+			, date_str
 #endif
 		);
 	}
@@ -1281,6 +1290,7 @@ static void send_headers_and_exit(int responseNum) NORETURN;
 static void send_headers_and_exit(int responseNum)
 {
 	IF_FEATURE_HTTPD_GZIP(content_gzip = 0;)
+	file_size = -1; /* no Last-Modified:, ETag:, Content-Length: */
 	send_headers(responseNum);
 	log_and_exit();
 }
@@ -1657,8 +1667,7 @@ static void send_cgi_and_exit(
 		script = last_slash;
 		if (script != url) { /* paranoia */
 			*script = '\0';
-			if (chdir(url + 1) != 0) {
-				bb_perror_msg("can't change directory to '%s'", url + 1);
+			if (chdir_or_warn(url + 1) != 0) {
 				goto error_execing_cgi;
 			}
 			// not needed: *script = '/';
@@ -1869,14 +1878,21 @@ static NOINLINE void send_file_and_exit(const char *url, int what)
 		send_headers(HTTP_OK);
 #if ENABLE_FEATURE_USE_SENDFILE
 	{
-		off_t offset = (range_start < 0) ? 0 : range_start;
+		off_t offset;
+# if ENABLE_FEATURE_HTTPD_RANGES
+		if (range_start < 0)
+			range_start = 0;
+		offset = range_start;
+# else
+		offset = 0;
+# endif
 		while (1) {
 			/* sz is rounded down to 64k */
 			ssize_t sz = MAXINT(ssize_t) - 0xffff;
 			IF_FEATURE_HTTPD_RANGES(if (sz > range_len) sz = range_len;)
 			count = sendfile(STDOUT_FILENO, fd, &offset, sz);
 			if (count < 0) {
-				if (offset == range_start)
+				if (offset == range_start) /* was it the very 1st sendfile? */
 					break; /* fall back to read/write loop */
 				goto fin;
 			}
@@ -2389,50 +2405,52 @@ static void handle_incoming_and_exit(const len_and_sockaddr *fromAddr)
 			send_headers_and_exit(HTTP_FORBIDDEN);
 		}
 		cgi_type = CGI_NORMAL;
-	}
+	} /* why "else": do not check "cgi-bin/SCRIPT/something" for cases below: */
+	else
 #endif
-
-	if (urlp[-1] == '/') {
-		/* When index_page string is appended to <dir>/ URL, it overwrites
-		 * the query string. If we fall back to call /cgi-bin/index.cgi,
-		 * query string would be lost and not available to the CGI.
-		 * Work around it by making a deep copy.
-		 */
-		if (ENABLE_FEATURE_HTTPD_CGI)
-			g_query = xstrdup(g_query); /* ok for NULL too */
-		strcpy(urlp, index_page);
-	}
-	if (stat(tptr, &sb) == 0) {
-		/* If URL is a directory with no slash, set up
-		 * "HTTP/1.1 302 Found" "Location: /dir/" reply */
-		if (urlp[-1] != '/' && S_ISDIR(sb.st_mode)) {
-			found_moved_temporarily = urlcopy;
-		} else {
+	{
+		if (urlp[-1] == '/') {
+			/* When index_page string is appended to <dir>/ URL, it overwrites
+			 * the query string. If we fall back to call /cgi-bin/index.cgi,
+			 * query string would be lost and not available to the CGI.
+			 * Work around it by making a deep copy.
+			 */
+			if (ENABLE_FEATURE_HTTPD_CGI)
+				g_query = xstrdup(g_query); /* ok for NULL too */
+			strcpy(urlp, index_page);
+		}
+		if (stat(tptr, &sb) == 0) {
+			/* If URL is a directory with no slash, set up
+			 * "HTTP/1.1 302 Found" "Location: /dir/" reply */
+			if (urlp[-1] != '/' && S_ISDIR(sb.st_mode)) {
+				found_moved_temporarily = urlcopy;
+			} else {
 #if ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR
-			char *suffix = strrchr(tptr, '.');
-			if (suffix) {
-				Htaccess *cur;
-				for (cur = script_i; cur; cur = cur->next) {
-					if (strcmp(cur->before_colon + 1, suffix) == 0) {
-						cgi_type = CGI_INTERPRETER;
-						break;
+				char *suffix = strrchr(tptr, '.');
+				if (suffix) {
+					Htaccess *cur;
+					for (cur = script_i; cur; cur = cur->next) {
+						if (strcmp(cur->before_colon + 1, suffix) == 0) {
+							cgi_type = CGI_INTERPRETER;
+							break;
+						}
 					}
 				}
+#endif
+				file_size = sb.st_size;
+				last_mod = sb.st_mtime;
 			}
-#endif
-			file_size = sb.st_size;
-			last_mod = sb.st_mtime;
 		}
-	}
 #if ENABLE_FEATURE_HTTPD_CGI
-	else if (urlp[-1] == '/') {
-		/* It's a dir URL and there is no index.html */
-		/* Is there cgi-bin/index.cgi? */
-		if (access("/cgi-bin/index.cgi"+1, X_OK) != 0)
-			send_headers_and_exit(HTTP_NOT_FOUND); /* no */
-		cgi_type = CGI_INDEX;
-	}
+		else if (urlp[-1] == '/') {
+			/* It's a dir URL and there is no index.html */
+			/* Is there cgi-bin/index.cgi? */
+			if (access("/cgi-bin/index.cgi"+1, X_OK) != 0)
+				send_headers_and_exit(HTTP_NOT_FOUND); /* no */
+			cgi_type = CGI_INDEX;
+		}
 #endif
+	}
 
 #if ENABLE_FEATURE_HTTPD_BASIC_AUTH || ENABLE_FEATURE_HTTPD_CGI
 	/* check_user_passwd() would be confused by added .../index.html, truncate it */
@@ -2695,8 +2713,8 @@ static void mini_httpd_nommu(int server_socket, int argc, char **argv)
 			/* Run a copy of ourself in inetd mode */
 			re_exec(argv_copy);
 		}
-		argv_copy[0][0] &= 0x7f;
 		/* parent, or vfork failed */
+		argv_copy[0][0] &= 0x7f; /* undo re_rexec() damage */
 		close(n);
 	} /* while (1) */
 	/* never reached */
