@@ -11,16 +11,13 @@ struct client_data_t {
 	uint8_t client_mac[6];          /* Our mac address */
 	IF_FEATURE_UDHCP_PORT(uint16_t port;)
 	int ifindex;                    /* Index number of the interface to use */
+	uint32_t xid;
 	uint8_t opt_mask[256 / 8];      /* Bitmask of options to send (-O option) */
 // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ TODO: DHCPv6 has 16-bit option numbers
 	const char *interface;          /* The name of the interface to use */
 	char *pidfile;                  /* Optionally store the process ID */
 	const char *script;             /* User script to run at dhcp events */
 	struct option_set *options;     /* list of DHCP options to send to server */
-	uint8_t *clientid;              /* Optional client id to use */
-	uint8_t *vendorclass;           /* Optional vendor class-id to use */
-	uint8_t *hostname;              /* Optional hostname to use */
-	uint8_t *fqdn;                  /* Optional fully qualified domain name to use */
 	llist_t *envp;                  /* list of DHCP options used for env vars */
 
 	unsigned first_secs;
