@@ -112,7 +112,7 @@ int dumpleases_main(int argc UNUSED_PARAM, char **argv)
 			printf("%02u:%02u:%02u\n", h, m, (unsigned)expires);
 		} else { /* -a */
 			time_t t = expires_abs;
-			fputs(ctime(&t), stdout);
+			fputs_stdout(ctime(&t));
 		}
 	}
 	/* close(fd); */
