@@ -619,17 +619,15 @@ static NOINLINE void display_process_list(int lines_rem, int scr_width)
 	unsigned busy_jifs;
 #endif
 
-	/* what info of the processes is shown */
-	printf(OPT_BATCH_MODE ? "%.*s" : ESC"[7m" "%.*s" ESC"[m", scr_width,
-		"  PID  PPID USER     STAT   VSZ %VSZ"
-		IF_FEATURE_TOP_SMP_PROCESS(" CPU")
-		IF_FEATURE_TOP_CPU_USAGE_PERCENTAGE(" %CPU")
-		" COMMAND");
-	lines_rem--;
-
 #if ENABLE_FEATURE_TOP_DECIMALS
 # define UPSCALE 1000
-# define CALC_STAT(name, val) div_t name = div((val), 10)
+typedef struct { unsigned quot, rem; } bb_div_t;
+/* Used to have "div_t name = div((val), 10)" here
+ * (IOW: intended to use libc-compatible way to divide and use
+ * both result and remainder, but musl does not inline div()...)
+ * Oh well. Modern compilers detect "N/d, N%d" idiom by themselves:
+ */
+# define CALC_STAT(name, val) bb_div_t name = { (val) / 10, (val) % 10 }
 # define SHOW_STAT(name) name.quot, '0'+name.rem
 # define FMT "%3u.%c"
 #else
@@ -638,6 +636,15 @@ static NOINLINE void display_process_list(int lines_rem, int scr_width)
 # define SHOW_STAT(name) name
 # define FMT "%4u%%"
 #endif
+
+	/* what info of the processes is shown */
+	printf(OPT_BATCH_MODE ? "%.*s" : ESC"[7m" "%.*s" ESC"[m", scr_width,
+		"  PID  PPID USER     STAT   VSZ %VSZ"
+		IF_FEATURE_TOP_SMP_PROCESS(" CPU")
+		IF_FEATURE_TOP_CPU_USAGE_PERCENTAGE(" %CPU")
+		" COMMAND");
+	lines_rem--;
+
 	/*
 	 * %VSZ = s->vsz/MemTotal
 	 */
@@ -689,6 +696,9 @@ static NOINLINE void display_process_list(int lines_rem, int scr_width)
 		lines_rem = ntop - G_scroll_ofs;
 	s = top + G_scroll_ofs;
 	while (--lines_rem >= 0) {
+		int n;
+		char *ppu;
+		char ppubuf[sizeof(int)*3 * 2 + 12];
 		char vsz_str_buf[8];
 		unsigned col;
 
@@ -699,12 +709,39 @@ static NOINLINE void display_process_list(int lines_rem, int scr_width)
 
 		smart_ulltoa5(s->vsz, vsz_str_buf, " mgtpezy");
 		/* PID PPID USER STAT VSZ %VSZ [%CPU] COMMAND */
+		n = sprintf(ppubuf, "%5u %5u %-8.8s", s->pid, s->ppid, get_cached_username(s->uid));
+		ppu = ppubuf;
+		if (n != 6+6+8) {
+			/* Format PID PPID USER part into 6+6+8 chars:
+			 * shrink PID/PPID if possible, then truncate USER.
+			 * Tested on Linux 5.18.0:
+			 * sysctl kernel.pid_max=4194304 is the maximum allowed,
+			 * so PID and PPID are 7 chars wide at most.
+			 */
+			char *p, *pp;
+			if (*ppu == ' ') {
+				do {
+					ppu++, n--;
+					if (n == 6+6+8)
+						goto shortened;
+				} while (*ppu == ' ');
+			}
+			pp = p = skip_non_whitespace(ppu) + 1;
+			if (*p == ' ') {
+				do
+					p++, n--;
+				while (n != 6+6+8 && *p == ' ');
+				overlapping_strcpy(pp, p); /* shrink PPID */
+			}
+			ppu[6+6+8] = '\0'; /* truncate USER */
+		}
+ shortened:
 		col = snprintf(line_buf, scr_width,
-				"\n" "%5u%6u %-8.8s %s  %.5s" FMT
+				"\n" "%s %s  %.5s" FMT
 				IF_FEATURE_TOP_SMP_PROCESS(" %3d")
 				IF_FEATURE_TOP_CPU_USAGE_PERCENTAGE(FMT)
 				" ",
-				s->pid, s->ppid, get_cached_username(s->uid),
+				ppu,
 				s->state, vsz_str_buf,
 				SHOW_STAT(pmem)
 				IF_FEATURE_TOP_SMP_PROCESS(, s->last_seen_on_cpu)
@@ -823,11 +860,15 @@ static void display_topmem_header(int scr_width, int *lines_rem_p)
 	(*lines_rem_p) -= 3;
 }
 
-static void ulltoa6_and_space(unsigned long long ul, char buf[6])
+/* see http://en.wikipedia.org/wiki/Tera */
+static void ulltoa5_and_space(unsigned long long ul, char buf[6])
 {
-	/* see http://en.wikipedia.org/wiki/Tera */
 	smart_ulltoa5(ul, buf, " mgtpezy")[0] = ' ';
 }
+static void ulltoa4_and_space(unsigned long long ul, char buf[5])
+{
+	smart_ulltoa4(ul, buf, " mgtpezy")[0] = ' ';
+}
 
 static NOINLINE void display_topmem_process_list(int lines_rem, int scr_width)
 {
@@ -852,14 +893,25 @@ static NOINLINE void display_topmem_process_list(int lines_rem, int scr_width)
 		lines_rem = ntop - G_scroll_ofs;
 	while (--lines_rem >= 0) {
 		/* PID VSZ VSZRW RSS (SHR) DIRTY (SHR) COMMAND */
-		ulltoa6_and_space(s->pid     , &line_buf[0*6]);
-		ulltoa6_and_space(s->vsz     , &line_buf[1*6]);
-		ulltoa6_and_space(s->vszrw   , &line_buf[2*6]);
-		ulltoa6_and_space(s->rss     , &line_buf[3*6]);
-		ulltoa6_and_space(s->rss_sh  , &line_buf[4*6]);
-		ulltoa6_and_space(s->dirty   , &line_buf[5*6]);
-		ulltoa6_and_space(s->dirty_sh, &line_buf[6*6]);
-		ulltoa6_and_space(s->stack   , &line_buf[7*6]);
+		int n = sprintf(line_buf, "%5u ", s->pid);
+		if (n > 7) {
+			/* PID is 7 chars long (up to 4194304) */
+			ulltoa4_and_space(s->vsz  , &line_buf[8]);
+			ulltoa4_and_space(s->vszrw, &line_buf[8+5]);
+			/* the next field (RSS) starts at 8+10 = 3*6 */
+		} else {
+			if (n == 7) /* PID is 6 chars long */
+				ulltoa4_and_space(s->vsz, &line_buf[7]);
+				/* the next field (VSZRW) starts at 7+5 = 2*6 */
+			else /* PID is 5 chars or less */
+				ulltoa5_and_space(s->vsz, &line_buf[6]);
+			ulltoa5_and_space(s->vszrw, &line_buf[2*6]);
+		}
+		ulltoa5_and_space(s->rss     , &line_buf[3*6]);
+		ulltoa5_and_space(s->rss_sh  , &line_buf[4*6]);
+		ulltoa5_and_space(s->dirty   , &line_buf[5*6]);
+		ulltoa5_and_space(s->dirty_sh, &line_buf[6*6]);
+		ulltoa5_and_space(s->stack   , &line_buf[7*6]);
 		line_buf[8*6] = '\0';
 		if (scr_width > (int)MIN_WIDTH) {
 			read_cmdline(&line_buf[8*6], scr_width - MIN_WIDTH, s->pid, s->comm);
@@ -913,7 +965,7 @@ static unsigned handle_input(unsigned scan_mask, duration_t interval)
 	while (1) {
 		int32_t c;
 
-		c = read_key(STDIN_FILENO, G.kbd_input, interval * 1000);
+		c = safe_read_key(STDIN_FILENO, G.kbd_input, interval * 1000);
 		if (c == -1 && errno != EAGAIN) {
 			/* error/EOF */
 			option_mask32 |= OPT_EOF;
