@@ -28,7 +28,7 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 /* Busyboxed by Denys Vlasenko <vda.linux@googlemail.com> */
 
 //config:config RUNSV
-//config:	bool "runsv (7.8 kb)"
+//config:	bool "runsv (8.2 kb)"
 //config:	default y
 //config:	help
 //config:	runsv starts and monitors a service and optionally an appendant log
@@ -58,11 +58,19 @@ static void gettimeofday_ns(struct timespec *ts)
 #else
 static void gettimeofday_ns(struct timespec *ts)
 {
-	BUILD_BUG_ON(sizeof(struct timeval) != sizeof(struct timespec));
-	BUILD_BUG_ON(sizeof(((struct timeval*)ts)->tv_usec) != sizeof(ts->tv_nsec));
-	/* Cheat */
-	gettimeofday((void*)ts, NULL);
-	ts->tv_nsec *= 1000;
+	if (sizeof(struct timeval) == sizeof(struct timespec)
+	 && sizeof(((struct timeval*)ts)->tv_usec) == sizeof(ts->tv_nsec)
+	) {
+		/* Cheat */
+		xgettimeofday((void*)ts);
+		ts->tv_nsec *= 1000;
+	} else {
+		/* For example, musl has "incompatible" layouts */
+		struct timeval tv;
+		xgettimeofday(&tv);
+		ts->tv_sec = tv.tv_sec;
+		ts->tv_nsec = tv.tv_usec * 1000;
+	}
 }
 #endif
 
@@ -141,17 +149,18 @@ static void warn_cannot(const char *m)
 	warn2_cannot(m, "");
 }
 
-static void s_child(int sig_no UNUSED_PARAM)
+/* SIGCHLD/TERM handler is reentrancy-safe because they are unmasked
+ * only over poll() call, not over memory allocations
+ * or printouts. Do not need to save/restore errno either,
+ * as poll() error is not checked there.
+ */
+static void s_chld_term(int sig_no)
 {
+	if (sig_no == SIGTERM)
+		sigterm = 1;
 	write(selfpipe.wr, "", 1);
 }
 
-static void s_term(int sig_no UNUSED_PARAM)
-{
-	sigterm = 1;
-	write(selfpipe.wr, "", 1); /* XXX */
-}
-
 static int open_trunc_or_warn(const char *name)
 {
 	/* Why O_NDELAY? */
@@ -372,11 +381,14 @@ static void startservice(struct svdir *s)
 				xdup2(logpipe.wr, 1);
 			}
 		}
-		/* Non-ignored signals revert to SIG_DFL on exec anyway */
-		/*bb_signals(0
-			+ (1 << SIGCHLD)
-			+ (1 << SIGTERM)
-			, SIG_DFL);*/
+		/* Non-ignored signals revert to SIG_DFL on exec.
+		 * But we can get signals BEFORE execl(), unlikely as that may be.
+		 * SIGCHLD is safe (would merely write to selfpipe),
+		 * but SIGTERM would set sigterm = 1 (with vfork, we affect parent).
+		 * Avoid that.
+		 */
+		/*signal(SIGCHLD, SIG_DFL);*/
+		signal(SIGTERM, SIG_DFL);
 		sig_unblock(SIGCHLD);
 		sig_unblock(SIGTERM);
 		execv(arg[0], (char**) arg);
@@ -503,9 +515,15 @@ int runsv_main(int argc UNUSED_PARAM, char **argv)
 	ndelay_on(selfpipe.wr);
 
 	sig_block(SIGCHLD);
-	bb_signals_recursive_norestart(1 << SIGCHLD, s_child);
 	sig_block(SIGTERM);
-	bb_signals_recursive_norestart(1 << SIGTERM, s_term);
+	/* No particular reason why we don't set SA_RESTART
+	 * (poll() wouldn't restart regardless of that flag),
+	 * we just follow what runit-2.1.2 does:
+	 */
+	bb_signals_norestart(0
+			+ (1 << SIGCHLD)
+			+ (1 << SIGTERM)
+			, s_chld_term);
 
 	xchdir(dir);
 	/* bss: svd[0].pid = 0; */
@@ -617,6 +635,7 @@ int runsv_main(int argc UNUSED_PARAM, char **argv)
 		sig_unblock(SIGTERM);
 		sig_unblock(SIGCHLD);
 		poll(x, 2 + haslog, 3600*1000);
+		/* NB: signal handlers can trash errno of poll() */
 		sig_block(SIGTERM);
 		sig_block(SIGCHLD);
 
@@ -651,7 +670,7 @@ int runsv_main(int argc UNUSED_PARAM, char **argv)
 				gettimeofday_ns(&svd[0].start);
 				update_status(&svd[0]);
 				if (LESS(svd[0].start.tv_sec, deadline))
-					sleep(1);
+					sleep1();
 			}
 			if (haslog) {
 				if (child == svd[1].pid) {
@@ -664,7 +683,7 @@ int runsv_main(int argc UNUSED_PARAM, char **argv)
 					gettimeofday_ns(&svd[1].start);
 					update_status(&svd[1]);
 					if (LESS(svd[1].start.tv_sec, deadline))
-						sleep(1);
+						sleep1();
 				}
 			}
 		} /* for (;;) */
@@ -681,7 +700,7 @@ int runsv_main(int argc UNUSED_PARAM, char **argv)
 
 		if (svd[0].sd_want == W_EXIT && svd[0].state == S_DOWN) {
 			if (svd[1].pid == 0)
-				_exit(EXIT_SUCCESS);
+				_exit_SUCCESS();
 			if (svd[1].sd_want != W_EXIT) {
 				svd[1].sd_want = W_EXIT;
 				/* stopservice(&svd[1]); */
