@@ -31,7 +31,7 @@ int load_policy_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 
 	rc = selinux_mkload_policy(1);
 	if (rc < 0) {
-		bb_perror_msg_and_die("can't load policy");
+		bb_simple_perror_msg_and_die("can't load policy");
 	}
 
 	return 0;
