@@ -9,14 +9,14 @@
  * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
 //config:config HEXDUMP
-//config:	bool "hexdump (8.6 kb)"
+//config:	bool "hexdump (8.7 kb)"
 //config:	default y
 //config:	help
 //config:	The hexdump utility is used to display binary data in a readable
 //config:	way that is comparable to the output from most hex editors.
 //config:
 //config:config HD
-//config:	bool "hd (7.8 kb)"
+//config:	bool "hd (8.3 kb)"
 //config:	default y
 //config:	help
 //config:	hd is an alias to hexdump -C.
@@ -72,14 +72,20 @@ static void bb_dump_addfile(dumper_t *dumper, char *name)
 }
 
 static const char *const add_strings[] ALIGN_PTR = {
-	"\"%07.7_ax \"16/1 \"%03o \"\"\n\"",   /* b */
-	"\"%07.7_ax \"16/1 \"%3_c \"\"\n\"",   /* c */
-	"\"%07.7_ax \"8/2 \"  %05u \"\"\n\"",  /* d */
-	"\"%07.7_ax \"8/2 \" %06o \"\"\n\"",   /* o */
-	"\"%07.7_ax \"8/2 \"   %04x \"\"\n\"", /* x */
+	"16/1 \" %03o"  , /* b */
+	"16/1 \" %3_c"  , /* c */
+	"8/2 \"   %05u" , /* d */
+	"8/2 \"  %06o"  , /* o */
+	"8/2 \"    %04x", /* x */
 };
 
-static const char add_first[] ALIGN1 = "\"%07.7_Ax\n\"";
+static void add_format(dumper_t *dumper, const char *fmt)
+{
+	char fmtbuf[sizeof("\"%07_ax\"" "%s\"" "\"\n\"") + 16];
+	sprintf(fmtbuf, "\"%%07_ax\"" "%s\"" "\"\n\"", fmt);
+	bb_dump_add(dumper, "\"%07_Ax\n\"");
+	bb_dump_add(dumper, fmtbuf);
+}
 
 static const char hexdump_opts[] ALIGN1 = "bcdoxCe:f:n:s:v";
 
@@ -104,15 +110,14 @@ int hexdump_main(int argc, char **argv)
 		if (!p)
 			bb_show_usage();
 		if ((p - hexdump_opts) < 5) {
-			bb_dump_add(dumper, add_first);
-			bb_dump_add(dumper, add_strings[(int)(p - hexdump_opts)]);
+			add_format(dumper, add_strings[(int)(p - hexdump_opts)]);
 		}
 		/* Save a little bit of space below by omitting the 'else's. */
 		if (ch == 'C') {
  hd_applet:
-			bb_dump_add(dumper, "\"%08.8_Ax\n\""); // final address line after dump
-			//------------------- "address  "   8 * "xx "    "  "  8 * "xx "
-			bb_dump_add(dumper, "\"%08.8_ax  \"8/1 \"%02x \"\"  \"8/1 \"%02x \"");
+			bb_dump_add(dumper, "\"%08_Ax\n\""); // final address line after dump
+			//------------------- "address "   8 * " xx"  " "  8 * " xx"
+			bb_dump_add(dumper, "\"%08_ax \"8/1 \" %02x\"\" \"8/1 \" %02x\"");
 			//------------------- "  |ASCII...........|\n"
 			bb_dump_add(dumper, "\"  |\"16/1 \"%_p\"\"|\n\"");
 		}
@@ -139,8 +144,7 @@ int hexdump_main(int argc, char **argv)
 	}
 
 	if (!dumper->fshead) {
-		bb_dump_add(dumper, add_first);
-		bb_dump_add(dumper, "\"%07.7_ax \"8/2 \"%04x \"\"\n\"");
+		add_format(dumper, "8/2 \" %04x");
 	}
 
 	argv += optind;
