@@ -6,7 +6,7 @@
  * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 //config:config REV
-//config:	bool "rev (4.4 kb)"
+//config:	bool "rev (4.6 kb)"
 //config:	default y
 //config:	help
 //config:	Reverse lines of a file or files.
@@ -51,7 +51,7 @@ static void strrev(CHAR_T *s, int len)
 int rev_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int rev_main(int argc UNUSED_PARAM, char **argv)
 {
-	int retval;
+	exitcode_t retval;
 	size_t bufsize;
 	char *buf;
 
